@@ -5,7 +5,7 @@
  * This file defines Horde's core API interface. Other core Horde libraries
  * can interact with Mnemo through this API.
  *
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL). If you
  * did not receive this file, see http://www.horde.org/licenses/apache.
@@ -15,7 +15,7 @@
 
 /* Determine the base directories. */
 if (!defined('MNEMO_BASE')) {
-    define('MNEMO_BASE', __DIR__ . '/..');
+    define('MNEMO_BASE', realpath(__DIR__ . '/..'));
 }
 
 if (!defined('HORDE_BASE')) {
@@ -24,7 +24,7 @@ if (!defined('HORDE_BASE')) {
     if (file_exists(MNEMO_BASE . '/config/horde.local.php')) {
         include MNEMO_BASE . '/config/horde.local.php';
     } else {
-        define('HORDE_BASE', MNEMO_BASE . '/..');
+        define('HORDE_BASE', realpath(MNEMO_BASE . '/..'));
     }
 }
 
@@ -43,7 +43,7 @@ class Mnemo_Application extends Horde_Registry_Application
 
     /**
      */
-    public $version = 'H5 (4.1.2)';
+    public $version = 'H5 (4.2.1)';
 
     /**
      * Global variables defined:
@@ -51,6 +51,17 @@ class Mnemo_Application extends Horde_Registry_Application
      */
     protected function _init()
     {
+        /* For now, autoloading the Content_* classes depend on there being a
+         * registry entry for the 'content' application that contains at least
+         * the fileroot entry. */
+        $GLOBALS['injector']->getInstance('Horde_Autoloader')
+            ->addClassPathMapper(
+                new Horde_Autoloader_ClassPathMapper_Prefix('/^Content_/', $GLOBALS['registry']->get('fileroot', 'content') . '/lib/'));
+
+        if (!class_exists('Content_Tagger')) {
+            throw new Horde_Exception(_("The Content_Tagger class could not be found. Make sure the Content application is installed."));
+        }
+
         Mnemo::initialize();
     }
 
@@ -124,9 +135,15 @@ class Mnemo_Application extends Horde_Registry_Application
             ),
         );
         foreach (Mnemo::listNotepads() as $name => $notepad) {
+            $url->add(array(
+                'display_notepad' => $name,
+                'actionID' => in_array($name, $GLOBALS['display_notepads'])
+                    ? 'remove_displaylist'
+                    : 'add_displaylist'
+            ));
             $row = array(
                 'selected' => in_array($name, $GLOBALS['display_notepads']),
-                'url' => $url->add('display_notepad', $name),
+                'url' => $url,
                 'label' => Mnemo::getLabel($notepad),
                 'color' => $notepad->get('color') ?: '#dddddd',
                 'edit' => $edit->add('n', $notepad->getName()),
@@ -161,7 +178,7 @@ class Mnemo_Application extends Horde_Registry_Application
     public function topbarCreate(Horde_Tree_Renderer_Base $tree, $parent = null,
                                  array $params = array())
     {
-        $add = Horde::url('memo.php')->add('actionID', 'add_memo');
+        $add = Horde::url('memo.php', true)->add('actionID', 'add_memo');
 
         $tree->addNode(array(
             'id' => $parent . '__new',
@@ -213,14 +230,14 @@ class Mnemo_Application extends Horde_Registry_Application
             try {
                 $driver->deleteAll();
             } catch (Mnemo_Exception $e) {
-                Horde::logMessage($e, 'NOTICE');
+                Horde::log($e, 'NOTICE');
                 $error = true;
             }
 
             try {
                 $GLOBALS['mnemo_shares']->removeShare($share);
             } catch (Horde_Share_Exception $e) {
-                Horde::logMessage($e, 'NOTICE');
+                Horde::log($e, 'NOTICE');
                 $error = true;
             }
         }
@@ -232,7 +249,7 @@ class Mnemo_Application extends Horde_Registry_Application
                 $share->removeUser($user);
             }
         } catch (Horde_Share_Exception $e) {
-            Horde::logMessage($e, 'NOTICE');
+            Horde::log($e, 'NOTICE');
             $error = true;
         }
 
@@ -265,6 +282,7 @@ class Mnemo_Application extends Horde_Registry_Application
                         $note['memolist_id'],
                         $nore['uid']
                     );
+                    $note['tags'] = implode(',', $note['tags']);
                     if ($note['body'] instanceof Mnemo_Exception) {
                         $note['body'] = $note['body']->getMessage();
                     }
