@@ -1,6 +1,6 @@
 <?php
 /**
- * Copyright 2001-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2001-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL). If you
  * did not receive this file, see http://www.horde.org/licenses/apache.
@@ -20,11 +20,6 @@ class Mnemo
      */
     const SORT_DESC = 0;
 
-    /**
-     * Sort by memo category.
-     */
-    const SORT_CATEGORY = 1;
-
     /**
      * Sort by notepad.
      */
@@ -60,8 +55,7 @@ class Mnemo
      * also sort the resulting list, if requested.
      *
      * @param constant $sortby   The field by which to sort. (self::SORT_DESC,
-     *                           self::SORT_CATEGORY, self::SORT_NOTEPAD,
-     *                           self::SORT_MOD_DATE)
+     *                           self::SORT_NOTEPAD, self::SORT_MOD_DATE)
      * @param constant $sortdir  The direction by which to sort.
      *                           (self::SORT_ASC, self::SORT_DESC)
      *
@@ -78,7 +72,6 @@ class Mnemo
         /* Sort the memo list. */
         $sort_functions = array(
             self::SORT_DESC => 'ByDesc',
-            self::SORT_CATEGORY => 'ByCategory',
             self::SORT_NOTEPAD => 'ByNotepad',
             self::SORT_MOD_DATE => 'ByModDate'
         );
@@ -188,7 +181,7 @@ class Mnemo
                           'attributes' => $owneronly ? $GLOBALS['registry']->getAuth() : null,
                           'sort_by' => 'name'));
             } catch (Horde_Share_Exception $e) {
-                Horde::logMessage($e->getMessage(), 'ERR');
+                Horde::log($e->getMessage(), 'ERR');
                 return array();
             }
         } else {
@@ -199,7 +192,7 @@ class Mnemo
                           'attributes' => $GLOBALS['registry']->getAuth(),
                           'sort_by' => 'name'));
             } catch (Horde_Share_Exception $e) {
-                Horde::logMessage($e);
+                Horde::log($e);
                 return array();
             }
             $display_notepads = @unserialize($GLOBALS['prefs']->getValue('display_notepads'));
@@ -212,7 +205,7 @@ class Mnemo
                         }
                     } catch (Horde_Exception_NotFound $e) {
                     } catch (Horde_Share_Exception $e) {
-                        Horde::logMessage($e);
+                        Horde::log($e);
                         return array();
                     }
                 }
@@ -244,13 +237,14 @@ class Mnemo
             ->getInstance('Mnemo_Factory_Notepads')
             ->create()
             ->getDefaultShare();
-        if (isset($notepads[$default_notepad])) {
-            $GLOBALS['prefs']->setValue('default_notepad', $default_notepad);
-            return $default_notepad;
+
+        if (!isset($notepads[$default_notepad])) {
+            reset($notepads);
+            $default_notepad = key($notepads);
         }
 
-        reset($notepads);
-        return key($notepads);
+        $GLOBALS['prefs']->setValue('default_notepad', $default_notepad);
+        return $default_notepad;
     }
 
     /**
@@ -321,35 +315,6 @@ class Mnemo
         return self::_sortByDesc($b, $a);
     }
 
-    /**
-     * Comparison function for sorting notes by category.
-     *
-     * @param array $a  Note one.
-     * @param array $b  Note two.
-     *
-     * @return integer  1 if note one is greater, -1 if note two is greater; 0
-     *                  if they are equal.
-     */
-    protected static function _sortByCategory($a, $b)
-    {
-        return strcoll($a['category'] ? $a['category'] : _("Unfiled"),
-                       $b['category'] ? $b['category'] : _("Unfiled"));
-    }
-
-    /**
-     * Comparison function for reverse sorting notes by category.
-     *
-     * @param array $a  Note one.
-     * @param array $b  Note two.
-     *
-     * @return integer  -1 if note one is greater, 1 if note two is greater; 0
-     *                  if they are equal.
-     */
-    protected static function _rsortByCategory($a, $b)
-    {
-        return self::_sortByCategory($b, $a);
-    }
-
     /**
      * Comparison function for sorting notes by notepad name.
      *
@@ -449,12 +414,11 @@ class Mnemo
      */
     public static function hasPermission($permission)
     {
-        global $perms;
+        $perms = $GLOBALS['injector']->getInstance('Horde_Perms');
 
         if (!$perms->exists('mnemo:' . $permission)) {
             return true;
         }
-
         $allowed = $perms->getPermissions('mnemo:' . $permission, $GLOBALS['registry']->getAuth());
         if (is_array($allowed)) {
             switch ($permission) {
@@ -513,15 +477,18 @@ class Mnemo
         // user doesn't have any selected notepads for view then fall
         // back to some available notepad.
         $GLOBALS['display_notepads'] = unserialize($GLOBALS['prefs']->getValue('display_notepads'));
-        if (($notepadId = Horde_Util::getFormData('display_notepad')) !== null) {
-            if (is_array($notepadId)) {
-                $GLOBALS['display_notepads'] = $notepadId;
-            } else {
+        if (($actionID = Horde_Util::getFormData('actionID')) !== null) {
+            $notepadId = Horde_Util::getFormData('display_notepad');
+            switch ($actionID) {
+            case 'add_displaylist':
+                if (!in_array($notepadId, $GLOBALS['display_notepads'])) {
+                    $GLOBALS['display_notepads'][] = $notepadId;
+                }
+                break;
+            case 'remove_displaylist':
                 if (in_array($notepadId, $GLOBALS['display_notepads'])) {
                     $key = array_search($notepadId, $GLOBALS['display_notepads']);
                     unset($GLOBALS['display_notepads'][$key]);
-                } else {
-                    $GLOBALS['display_notepads'][] = $notepadId;
                 }
             }
         }
@@ -555,17 +522,41 @@ class Mnemo
     }
 
     /**
+     * Returns the notepads that should be used for syncing.
+     *
+     * @param boolean $prune  Remove notepads ids from the sync list that no
+     *                        longer exist. The values are pruned *after* the
+     *                        results are passed back to the client to give
+     *                        sync clients a chance to remove their entries.
+     *
+     * @return array  An array of notepad ids.
      */
-    static public function getCssStyle($category)
+    static public function getSyncNotepads($prune = false)
     {
-        $cManager = new Horde_Prefs_CategoryManager();
-        $colors = $cManager->colors();
-        if (!isset($colors[$category])) {
-            return '';
+        $haveRemoved = false;
+        $cs = unserialize($GLOBALS['prefs']->getValue('sync_notepads'));
+        if (!empty($cs)) {
+            if ($prune) {
+                $notepads =  self::listNotepads(true, Horde_Perms::EDIT);
+                $cscopy = array_flip($cs);
+                foreach ($cs as $c) {
+                    if (empty($notepads[$c])) {
+                        unset($cscopy[$c]);
+                        $haveRemoved = true;
+                    }
+                }
+                if ($haveRemoved) {
+                    $GLOBALS['prefs']->setValue('sync_notepads', serialize(array_flip($cscopy)));
+                }
+            }
+            return $cs;
         }
-        $fgColors = $cManager->fgColors();
 
-        return 'color:' . (isset($fgColors[$category]) ? $fgColors[$category] : $fgColors['_default_']) . ';' .
-            'background:' . $colors[$category] . ';';
+        if ($cs = self::getDefaultNotepad(Horde_Perms::EDIT)) {
+            return array($cs);
+        }
+
+        return array();
     }
+
 }
